require 'calc.rb'

class SensorExpressionProcessor < ExpressionProcessor
  # Let's just add all of the usual operators
  include ParentheticalGrouping           # ()
  include AssignmentOperator              # =
  include LastResultOperator              # $
  include ArithmeticOperators             # +-*/%^
  include BooleanOperators                # true false and or eq ne gt lt ge le not

  def add_expr(name, &block)
    @operators << Function.new(name, 0, 0) { yield block }
  end
end

class Sensor
  attr_reader :node, :thresholds, :name, :monitor

  def add_expr(name, &block)
    @expr.add_expr(name, &block)
  end

  def initialize(node, xmlsensor, name)
    @node = node
    @name = name
    @monitor = Array.new
    @expr = SensorExpressionProcessor.new
    @status = ["ok", nil]

    # Read thresholds...
    begin
	xmlsensor.elements["thresholds"].elements.each {|mon|
	    @monitor.push([
		      mon.attributes["expr"],
		      mon.attributes["status"],
		      mon.attributes["msg"],
		    ])
	}
    rescue
	begin
	    # Read defaults...
	    xmlsensor.root.elements["/config/defaults/" + xmlsensor.name + "/thresholds"].elements.each {|mon|
		@monitor.push([
		      mon.attributes["expr"],
		      mon.attributes["status"],
		      mon.attributes["msg"],
		    ])
	    }
	rescue
	    # No thresholds defined for this sensor.
	end
    end
  end

  def set_status(status, msg)
    if @status[0] != status or @status[1] != msg then
      dname = "#{@node.name}-#{@name}"
      # Consume any pending status from TS
      begin
	ScottyTNG.getTS.take(["SensorStatus", dname, nil], 0) while true
      rescue
      end

      # Write new status to TS
      ScottyTNG.getTS.write ["SensorStatus", dname, [Time.now.to_i, status, msg]]
      @status = [status, msg]

      puts "#{@node.name}/#{@name}: status '#{status}': #{msg}"
    end
  end

  def check_mon
    @monitor.each{|m|
      if @expr.eval(m[0]) then
	self.set_status(m[1], m[2])
	return
      end
    }
    self.set_status("ok", nil)
  end

  def self.create(node, xmlsensor)
    case xmlsensor.name
    when "rtt"
      return Sensors::RTT.new(node, xmlsensor)
    when "if"
      return Sensors::IF.new(node, xmlsensor)
    else
      puts "Could not find sensor class for:\n" + xmlsensor.to_s + "\n"
    end
  end
end
