#!/usr/bin/perl -w

use strict;

my $in = 0;
my @block;

my %nodes;
my %stripcharts;

while(<>) {
    chomp;

    if ($_ eq '') {
	if ($in == 1) {
	    my $b = join('|', @block);

	    $b =~ /noupdate create (\S+) /;
	    my $type = $1;
	
	    if ($type eq 'NODE') {
		$b =~ /name \$\S+ (\S+|\{[^|]+\})\|/;
		my $name = $1;
		$b =~ /address \$\S+ (\S+)\|/;
		my $address = $1;

		$address = &hostname($address) unless($address =~ /^[\d.]+$/);

		${$nodes{$name}}{'name'} = $name;
		${$nodes{$name}}{'address'} = $address;
	    }
	    elsif ($type eq 'STRIPCHART' && $b =~ /round trip/) {
		$b =~ /name \$\S+ (\S+|\{[^|]+\})\|/;
		my $name = $1;
		$b =~ /address \$\S+ (\S+)\|/;
		my $address = $1;

		$address = &hostname($address) unless($address =~ /^[\d.]+$/);

		${$nodes{$name}}{'name'} = $name;
		${$nodes{$name}}{'address'} = $address;
	    }
	    elsif ($type eq 'STRIPCHART' && $b =~ /ifLoad/) {
		$b =~ /name \$(\S+) (\S+|\{[^|]+\})\|/;
		my $name = $1;
		my $strip = $2;

		push(@{${$nodes{$strip}}{'if'}}, $name);
	    }
	    elsif ($type eq 'INTERPRETER' && $b =~ /snmp_monitor\.tcl/) {
		while($b =~ /(strip[\w\d]+) (\d+)/) {
		    $stripcharts{$1} = $2;
		    $b =~ s/$1 $2//;
		}
	    }
	}

	$in = 1;
	@block = ();
	next;
    }

    push(@block, $_) if ($in == 1 && $_ ne '');
}

foreach my $name (keys %nodes) {
    print "	    <node name='$name' address='", ${$nodes{$name}}{'address'}, "'>
		<rtt/>\n";
    foreach my $strip (@{${$nodes{$name}}{'if'}}) {
	print "		<if idx='", $stripcharts{$strip}, "'/>\n";
    }
    print "	    </node>\n";
}

sub hostname {
  my (@bytes, @octets,
    $packedaddr,
    $raw_addr,
    $host_name,
    $ip
  );

  if($_[0] =~ /[a-zA-Z]/g) {
    $raw_addr = (gethostbyname($_[0]))[4];
    @octets = unpack("C4", $raw_addr);
    $host_name = join(".", @octets);
  } else {
    @bytes = split(/\./, $_[0]);
    $packedaddr = pack("C4",@bytes);
    $host_name = (gethostbyaddr($packedaddr, 2))[0];
  }

  return($host_name);
}
