#!/usr/bin/perl -w

use strict;

my $in = 0;
my @block;

my @nodes;
my @texts;
my @charts;
my %nnames;
my @links;
my %refs;

while(<>) {
    chomp;

    if ($_ eq '') {
	if ($in == 1) {
	    my $b = join('|', @block);

	    $b =~ /noupdate create (\S+) /;
	    my $type = $1;
	
	    if ($type eq 'NODE') {
		$b =~ /name (\$\S+) ([^|]+)\|/;
		my $old = $1;
		my $name = $2;

		$b =~ /move \$\S+ (\S+) (\S+)\|/;
		my $x = int($1);
		my $y = int($2);
		
		$nnames{$old} = $name;
		push(@nodes, "	<node name='$name' x='$x' y='$y'/>");
	    }
	    elsif ($type eq 'TEXT') {
		$b =~ /create TEXT \{([^}]+)\} /;
		my $text = $1;
		$b =~ /move \$\S+ (\S+) (\S+)\|/;
		my $x = int($1);
		my $y = int($2);
		$b =~ /color \$\S+ ([a-z]+)/i;
		my $color = $1;
		push(@texts, "	<text x='$x' y='$y' color='$color'>$text</text>");
	    }
	    elsif ($type eq 'LINK') {
		$b =~ /color \$\S+ ([a-z]+)/i;
		my $color = $1;
		$b =~ /create LINK (\S+) (\S+) /;
		push(@links, "	<link n1='".$nnames{$1}."' n2='".$nnames{$2}."' color='$color'/>");
	    }
	    elsif ($type eq 'STRIPCHART' && $b =~ /round trip time/) {
		$b =~ /name \$\S+ ([^|]+)\|/;
		my $node = $1;

		$b =~ /move \$\S+ (\S+) (\S+)\|/;
		my $x = int($1);
		my $y = int($2);

		push(@charts, "	<chart node='$node' sensor='rtt' x='$x' y='$y'/>");
	    }
	    elsif ($type eq 'STRIPCHART' && $b =~ /ifLoad/) {
		$b =~ /name \$\S+ ([^|]+)\|/;
		my $node = $1;

		$b =~ /move \$\S+ (\S+) (\S+)\|/;
		my $x = int($1);
		my $y = int($2);

		$b =~ /move (\$\S+) /;
		my $ref = $1;

		push(@charts, "	<chart node='$node' sensor='if[##$ref##]' x='$x' y='$y'/>");
	    }
	    elsif ($type eq 'INTERPRETER' && $b =~ /snmp_monitor\.tcl/) {
		while($b =~ /(strip[\w\d]+) (\d+)/) {
		    $refs{$1} = $2;
		    $b =~ s/$1 $2//;
		}
	    }
	}

	$in = 1;
	@block = ();
	next;
    }

    push(@block, $_) if ($in == 1 && $_ ne '');
}

print join("\n", @nodes), "\n\n";
print join("\n", @texts), "\n\n";
print join("\n", @links), "\n\n";

my $charts = join("\n", @charts);
foreach my $s (keys %refs) {
    $charts =~ s/##\$$s##/$refs{$s}/g;
}

print "$charts\n\n";
