#!/usr/bin/ruby -w
#
# $Id$
#
# The editor for Scotty TNG


require 'gtk2'
require 'cairo'
require 'libglade2'
require 'zlib'
require 'rexml/document'

begin
  Dir.chdir('/usr/share/scotty-tng')
rescue
end

class EditorGlade
  include GetText

  attr :glade

  def initialize(path_or_data, root = nil, domain = nil, localedir = nil, flag = GladeXML::FILE)
    bindtextdomain(domain, localedir, nil, "UTF-8")
    @glade = GladeXML.new(path_or_data, root, domain, localedir, flag) {|handler| method(handler)}
    @window_main = @glade.get_widget("window_main")
    @scrolledwindow_log = @glade.get_widget("scrolledwindow_log")
    @menu_view = @glade.get_widget("menu_view")

    # Show main window and startup splash
    Gtk::Window.auto_startup_notification = false
    @window_main.show
    Gtk::Window.auto_startup_notification = true

    @width = 320	# initial width of the view
    @height = 240	# initial height of the view

    @nodes = Hash.new	# hash of nodes on this view
    @charts = Hash.new	# hash of charts on this view

    @cwidth = 60
    @cheight = 35
    @cfoot = 10

    # Gtk::Layout is used to draw the view
    @layout = @glade.get_widget("layout_main")
    @layout_img = nil
    @layout_ct = nil
  end

  # Process pending Gtk events and return
  def iter
    while Gtk.events_pending? do
      Gtk.main_iteration
    end
  end

  # Print a loging message on screen
  def logtxt(text)
    @textview_log.buffer.insert(@textview_log.buffer.end_iter, text.to_s + "\n")
    iter

    @scrolledwindow_log.vadjustment.value = @scrolledwindow_log.vadjustment.upper -
      @scrolledwindow_log.vadjustment.page_size
    iter
  end

  # Initialize the Editor stuff
  def init_editor
    iter

    return 1
  end

  # Load a view from XML string
  def load_view(xview)
    @window_main.title = "Scotty TNG Editor [" + xview.attributes["title"] + "]"

    # Setup layout size
    @width = xview.attributes["width"].to_i
    @height = xview.attributes["height"].to_i

    @layout.set_size(@width, @height)
    @layout_img = Cairo::ImageSurface.new(Cairo::FORMAT_ARGB32, @width, @height)
    @layout_ct = Cairo::Context.new(@layout_img)

    # Add text elements
    xview.elements.each("text") { |text|
      label = Gtk::Label.new
      if text.attributes["color"] != nil then
	label.markup = "<span foreground='"+ text.attributes['color'] +"'>#{text.text}</span>"
      else
	label.text = text.text
      end
      @layout.put(label, text.attributes["x"].to_i, text.attributes["y"].to_i)
      label.show
    }

    # Add chart elements
    xview.elements.each("chart") { |chart|
      name = chart.attributes["node"] + "-" + chart.attributes["sensor"]

      img = Cairo::ImageSurface.new(Cairo::FORMAT_ARGB32, @cwidth, @cheight + @cfoot)
      ct = Cairo::Context.new(img)

      da = Gtk::DrawingArea.new
      da.set_size_request(@cwidth, @cheight + @cfoot)


      @charts[name] = {
	"node" => chart.attributes['node'],
	"x" => chart.attributes["x"].to_i,
	"y" => chart.attributes["y"].to_i,
	"sx" => chart.attributes["x"].to_i,
	"sy" => chart.attributes["y"].to_i,
	"img" => img,
	"ct" => ct,
	"gtk_da" => da,
      }

      da.signal_connect("expose_event") { |widget, arg0|
	draw_chart_data(ct, @charts[name])

	# Copy image to drawing area surface
	c = widget.window.create_cairo_context
	c.set_source(img, 0, 0)
	c.paint
      }

      @layout.put(da, @charts[name]["x"], @charts[name]["y"])
      da.signal_connect("size_allocate") { |widget, alloc|
	nx = @charts[name]["x"] - alloc.width/2
	ny = @charts[name]["y"] - alloc.height/2
	
	if @charts[name]["sx"] != nx or
	    @charts[name]["sy"] != ny then
          @charts[name]["sx"] = nx
	  @charts[name]["sy"] = ny
	  @layout.move(widget, nx, ny)
	end
      }

      da.show_all

      Gtk::Drag.highlight(da)
      Gtk::Drag.source_set(da, Gdk::Window::BUTTON1_MASK, [['reorder-elements', Gtk::Drag::TARGET_SAME_APP, 1]], Gdk::DragContext::ACTION_MOVE)
      #     # con = Gtk::Drag.begin(da, Gdk::Window::BUTTON1_MASK, [['reorder-elements', Gtk::Drag::TARGET_SAME_APP, 1]], Gdk::DragContext::ACTION_MOVE, Gdk::Event::DRAG_ENTER)
      # Gtk::Drag.set_icon(con, da, 0, 0)
    }

    # Add node elements
    xview.elements.each("node") { |node|
      x = node.attributes["x"].to_i
      y = node.attributes["y"].to_i

      vb = Gtk::VBox.new(false, 2)
      fn = node.attributes['icon']
      unless fn && fn.match(/^\w\w$/)
	img = Gtk::Image.new("res/rmbl.png")
      else
	img = Gtk::Image.new("res/" + fn.downcase + "bl.png")
      end
      vb.pack_start(img)
      eb = Gtk::EventBox.new
      label = Gtk::Label.new
      label.use_markup = true
      label.markup = "<small>#{node.attributes['name']}</small>"
      eb.add(label)
      vb.pack_start(eb)
      @layout.put(vb, x, y)
      vb.signal_connect("size_allocate") { |widget, alloc|
	nx = @nodes[node.attributes["name"]]["x"] - vb.size_request[0]/2
	ny = @nodes[node.attributes["name"]]["y"] - img.size_request[1]/2

	if @nodes[node.attributes["name"]]["sx"] != nx or
	    @nodes[node.attributes["name"]]["sy"] != ny then
	  @nodes[node.attributes["name"]]["sx"] = nx
	  @nodes[node.attributes["name"]]["sy"] = ny
	  @layout.move(widget, nx, ny)
	end
      }
      vb.show_all

      @nodes[node.attributes["name"]] = {
	"x" => x,
	"y" => y,
	"sx" => x,
	"sy" => y,
	"gtk_vb" => vb,
	"gtk_img" => img,
      }
    }

    # Draw links between nodes on background
    xview.elements.each("link") { |link|
      set_ct_scol(@layout_ct, link.attributes["color"].to_s)
      n1 = link.attributes["n1"]
      n2 = link.attributes["n2"]

      @layout_ct.move_to(@nodes[n1]["x"], @nodes[n1]["y"])
      @layout_ct.line_to(@nodes[n2]["x"], @nodes[n2]["y"])
      @layout_ct.stroke
    }

    @layout.queue_draw
  end


  ##
  ## window_main
  ##

  # Draws the content of a chart
  def draw_frame(ct, chart)
    ct.set_source_rgb(0, 0, 0)
    ct.rectangle(0, 0, @cwidth, @cheight)
    ct.move_to(1, 9)
    #    ct.show_text(chart["opts"]["title"])
    ct.stroke

    ct.set_source_rgb(0.9, 0.9, 0.9)
    ct.rectangle(0, @cheight, @cwidth, @cfoot)
    ct.fill_preserve
    ct.stroke
  end

  def draw_chart_data(ct, chart)
    height = @cheight
    width = @cwidth

    ct.line_width = 1
    ct.set_font_size(9);

    # Clear chart
    ct.set_source_rgb(1, 1, 1)
    ct.rectangle(0, 0, width, height)
    ct.fill_preserve
    ct.stroke

    # Draw the lines
    hasdata = 0

    # Warn about missing data
    if hasdata == 0 then
      self.draw_frame(ct, chart)

      ct.set_font_size(8);
      ct.set_source_rgb(0.8, 0, 0)

      ct.move_to(1, @cheight + @cfoot - 2)
      ct.show_text("No data!")

      return
    end
    
    # Draw frame
    self.draw_frame(ct, chart)
  end

  # Set cairo context color by string
  def set_ct_scol(ct, color)
    begin
      ct.set_source_color(color)
    rescue
      case color.downcase
      when "red"
	ct.set_source_rgb(1, 0, 0)
      when "darkred"
	ct.set_source_rgb(0.7, 0, 0)
      when "green"
	ct.set_source_rgb(0, 1, 0)
      when "darkgreen"
	ct.set_source_rgb(0, 0.7, 0)
      when "blue"
	ct.set_source_rgb(0, 0, 1)
      when "darkblue"
	ct.set_source_rgb(0, 0, 0.7)
      else
	# Nothing matched, black is always good!
	ct.set_source_rgb(0, 0, 0)
      end
    end
  end

  # Formats a value for a given format (=[t]ime, [p]ercent, [b]ytes...)
  def format_data(format, val)
    case format
      # Time in ms
    when "t"
      return sprintf("%dms", val) if val < 1000
      return sprintf("%.1fs", val/1000)
      # Percent
    when "p"
      return sprintf("%d%%", val)
      # Bitrate
    when "r"
      val *= 8
      
      return sprintf("%.1fT", val / 1000000000000.0) if val > 100000000000
      return sprintf("%.1fG", val / 1000000000.0   ) if val > 100000000
      return sprintf("%.1fM", val / 1000000.0      ) if val > 100000
      return sprintf("%.1fk", val / 1000.0         ) if val > 100
      return sprintf("%d", val)
    when "b"
      return "yes" if val > 0
      return "no"
      # Anything else
    else
      return sprintf("%d", val)
    end
  end

  # Draw layout background (link lines)
  def layout_main_expose_event_cb(widget, arg0)
    if @layout_img then
      c = widget.bin_window.create_cairo_context
      c.set_source(@layout_img, 0, 0)
      c.paint
    end

    return false
  end

  # Terminate if main window is closed
  def window_main_delete_event_cb(widget, arg0)
    Gtk.main_quit
  end

  def on_quit1_activate(widget)
    Gtk.main_quit
  end

  def on_open1_activate(widget)
    parent_window = widget.toplevel
    # TODO
    dialog = Gtk::FileChooserDialog.new("Open File",
					parent_window,
					Gtk::FileChooser::
					ACTION_OPEN,
					nil,
					[Gtk::Stock::CANCEL, Gtk::Dialog::
					  RESPONSE_CANCEL],
					[Gtk::Stock::OPEN, Gtk::Dialog::
					  RESPONSE_ACCEPT])

    
    if dialog.run == Gtk::Dialog::RESPONSE_ACCEPT
      #puts "filename = #{dialog.filename}"
      f = File.new(dialog.filename)
      str = String.new
      begin
	while (line = f.readline)
	  str << line.chomp
	end
      rescue EOFError
	f.close
      end

      #      begin
      xml = REXML::Document.new(str)
      
      xml.elements.each("/views/view") { |xview|
	@menu_view.append(Gtk::MenuItem.new(xview.attributes["title"]))
	load_view(xview)
      }
      @menu_view.show_all

      #      rescue
      #	Gtk.main_quit
      #      end
    end
    dialog.destroy
  end

  def on_save1_activate(widget)
    # TODO
  end

  def on_save_as1_activate(widget)
    # TODO
  end

  def on_delete1_activate(widget)
    # TODO
  end

  def on_new1_activate(widget)
    # TODO
  end

  def on_copy1_activate(widget)
    # TODO
  end

  def on_cut1_activate(widget)
    # TODO
  end

  def on_paste1_activate(widget)
    # TODO
  end

  def on_about1_activate(widget)
    # TODO
    dialog_about = Gtk::AboutDialog.new
    dialog_about.show
  end

  def button_exit_clicked_cb(widget)
    dialog = widget.toplevel
    dialog.hide
  end
end

if EditorGlade.new("editor.glade").init_editor != 0 then
  Gtk.main
end
